import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;

public class PlaceLabel extends JLabel implements MouseListener, MouseMotionListener{
	Element place;
	int diameter = 40;
	int offset = 2;
	int offset2 = offset * 2;
	int textspace = 12;
	int scaling = 1;
      Dimension preferredSize = new Dimension(diameter+offset2,diameter+offset2);
	boolean beingMoved=false;
	int lastX, lastY;

    public PlaceLabel(Element place) {
	super("");
	setHorizontalAlignment(CENTER);
	this.place = place;
	Namespace ns = place.getNamespace();
	setText(place.getChild("initialMarking",ns).getChild("value",ns).getText());
	int x = scaling * Integer.parseInt(place.getChild("graphics",ns).getChild("position",ns).getAttributeValue("x"));
	int y = scaling * Integer.parseInt(place.getChild("graphics",ns).getChild("position",ns).getAttributeValue("y"));
	setBounds(x,y,diameter+offset2,diameter+offset2+textspace);
		// setLocation(x,y); // also works, but need to setBounds first

      addMouseMotionListener(this);
      addMouseListener(this);
    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
	  g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
       // Draws the circle.
	  g2.setStroke(new BasicStroke(2.0f));
	  g2.draw(new Ellipse2D.Double(offset, offset, diameter, diameter));
	  // Draws the name.
	  Namespace ns = place.getNamespace();
	  String name = place.getChild("name",ns).getChild("value",ns).getText();
	  g2.drawString(name,20,diameter+offset+textspace);

    }


    public Dimension getPreferredSize() {
        return preferredSize;
    }

	// function to help with debugging
	public void examineElement(Element e) {
		System.out.print("Examining element...\n");
		Namespace ns = e.getNamespace();
		java.util.List contents = e.getChildren();
		Iterator es= contents.iterator();
		while (es.hasNext()) {
			Element c = (Element) es.next();
			System.out.print("found element "+c.getName()+ " with namespace "+
							c.getNamespace().toString()+ "\n");
		}

	}


    // Handles the event of a user dragging the mouse while holding
    // down the mouse button.
    public void mouseDragged(MouseEvent e){
            if ( beingMoved ) {
	//	updateLocation(e); // does nasty flickering
		}
    }

    // Handles the event of the user pressing down the mouse button.
    public void mousePressed(MouseEvent e){
		beingMoved=true;
		lastX = getX() - e.getX() -offset; // don't really want -offset here, but
		lastY = getY() - e.getY() -offset; // it's useful while mouseDragged doesn't work
		updateLocation(e);
    }

    // Handles the event of a user releasing the mouse button.
    public void mouseReleased(MouseEvent e){
            if ( beingMoved ) {
		 	updateLocation(e);
			// Update DOM with new values
			Namespace ns = place.getNamespace();
			place.getChild("graphics",ns).getChild("position",ns).setAttribute("x",String.valueOf(getX()/scaling));
			place.getChild("graphics",ns).getChild("position",ns).setAttribute("y",String.valueOf(getY()/scaling));
			beingMoved=false;
		}
    }
       
     // This method is required by MouseListener.
     public void mouseMoved(MouseEvent e){}

     // These methods are required by MouseMotionListener.
     public void mouseClicked(MouseEvent e){}
     public void mouseExited(MouseEvent e){}
     public void mouseEntered(MouseEvent e){}
                         
     // Updates the coordinates representing the location of the current rectangle.
     public void updateLocation(MouseEvent e){
                 setLocation(lastX + e.getX(), lastY + e.getY());
			repaint();
     }

}

